# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'

#
# Default files where JAVA_HOME settings might be found in a WebLogic installation
#
DEFAULT_WLS_JAVA_HOME_FILES = [
  # Nodemanager
  '/nodemanager/nodemanager.properties',
  # bin
  '/bin/setNMJavaHome.sh',
  '/bin/setDomainEnv.sh',
  # init-info
  '/init-info/domain-info.xml',
  '/init-info/startscript.xml',
  '/init-info/tokenValue.properties',
  '/init-info/nodemanager-properties.xml',
  # config
  '/config/config.xml',
  '/config/config_bootstrap.xml',
  # rest
  '/backup_config/backup_config.xml',
  '/servers/domain_bak/config_prev/config.xml'
].freeze

#
# Type definition for managing the JAVA_HOME setting across various WebLogic
# configuration files within a specific domain or middleware home.
#
# It identifies the predominant JAVA_HOME setting across a default set of files
# and allows modification of this setting if necessary.
#
Puppet::Type.newtype(:wls_java_home) do
  include EasyType

  desc <<-DESC
  Ensure correct java home is present in WebLogic files in specified weblogic home.

  This type scans a predefined list of configuration files within a WebLogic
  home directory (`name`) to determine the currently configured `java_home`.
  If multiple distinct `java_home` paths are found, it attempts to determine the
  predominant one. If a single predominant path cannot be determined (e.g., a tie),
  it will raise an error.

  When the `java_home` property is managed (i.e., specified in the Puppet manifest),
  this type will replace occurrences of the *currently detected* `java_home` with
  the *desired* `java_home` in the default files and any specified `extra_files`.
  DESC

  # @!method self.domain_file
  # Returns the path to the file containing the list of managed WebLogic domains.
  # @return [String] The absolute path to the domain list file.
  def self.domain_file
    '/etc/wls_domains.dat'
  end

  #
  # This method is called by Puppet to discover existing instances of this resource type.
  # It reads the domain file, extracts the home path for each domain, and determines
  # the current `java_home` for that path using {#java_home_for}.
  #
  # @return [Array<Hash>] An array of hashes, where each hash represents a discovered
  #   resource instance with its `name` (the WebLogic home path) and detected `java_home`.
  #   Returns an empty array if the domain file doesn't exist.
  #
  to_get_raw_resources do
    return [] unless File.exist?(domain_file)

    file = File.read(domain_file)
    file.split.map do |domain|
      home = domain.split(':')[1]
      # Determine the single predominant Java home for this WLS home
      java_home = java_home_for(home)
      { :name => home, :java_home => java_home }
    end
  end

  # Determines the predominant JAVA_HOME setting for a given WebLogic home directory.
  # It scans the `DEFAULT_FILES` within that home directory.
  #
  # @param home [String] The root path of the WebLogic domain or middleware home.
  # @return [String, nil] The assessed predominant JAVA_HOME path, or nil if none
  #   could be definitively determined or found.
  # Renamed back from java_homes_for
  def self.java_home_for(home)
    # Use flat_map to get a single list of all detected java homes from default files
    java_homes = DEFAULT_WLS_JAVA_HOME_FILES.flat_map { |file_suffix| java_homes_in_file(File.join(home, file_suffix)) }.compact
    assess_homes(java_homes)
  end

  # Scans a single file for lines defining JAVA_HOME or JavaHome and returns the paths.
  # Excludes assignments where the path contains a '$' (likely a variable).
  #
  # @param file [String] The absolute path to the file to scan.
  # @return [Array<String>] An array of detected JAVA_HOME paths found in the file.
  #   Returns an empty array if the file doesn't exist or no valid paths are found.
  def self.java_homes_in_file(file)
    if File.exist?(file)
      Puppet.debug "Checking java_home in #{file}"
      content = File.read(file, :encoding => 'UTF-8')
      java_homes = content.scan(/JAVA_HOME="([^$@].*)"/)
      java_homes += content.scan(/JavaHome=(.*)/)
      java_homes.flatten!
      Puppet.debug "Found java_homes: #{java_homes.join(', ')}" unless java_homes.empty?
      java_homes
    else
      Puppet.debug "Skipping check of java_home in #{file}, because file not found."
      [] # Return empty array instead of nil for consistency
    end
  end

  # Replaces occurrences of the currently detected Java home path (`provider.java_home`)
  # with the desired Java home path (`java_home`) within a specific file.
  # Uses pattern matching to ensure replacements only occur in actual Java home settings.
  #
  # @param file [String] The absolute path to the file to modify.
  # @param warning [Boolean] If true, issue a warning if the file is not found;
  #   otherwise, use debug logging.
  # @return [void]
  # @api private
  # rubocop: disable Style/OptionalBooleanParameter
  def replace_java_home(file, warning = false)
    desired_java_home = self[:java_home] # Use self[:java_home] to get target value

    # Ensure we have a valid java_home to set
    return unless desired_java_home && !desired_java_home.empty?

    if File.exist?(file)
      begin
        Puppet.debug "Setting java_home in #{file} to: '#{desired_java_home}'"
        content = File.read(file, :encoding => 'UTF-8')

        # Track if any changes were made
        changes_made = false

        # Replace JAVA_HOME="any_value" with JAVA_HOME="new_value"
        content.gsub!(/(JAVA_HOME=")([^"$]*)(")/i) do
          changes_made = true
          "#{Regexp.last_match(1)}#{desired_java_home}#{Regexp.last_match(3)}"
        end

        # Replace unquoted JavaHome=any_value with JavaHome=new_value
        content.gsub!(/(JavaHome=)([^"\s$][^\s]*)(\s|$)/i) do
          changes_made = true
          "#{Regexp.last_match(1)}#{desired_java_home}#{Regexp.last_match(3)}"
        end

        # Only write the file if changes were actually made
        if changes_made
          File.write(file, content, :encoding => 'UTF-8')
          Puppet.debug "Successfully updated java_home in #{file}"
        else
          Puppet.debug "No matching java_home settings found to replace in #{file}"
        end
      rescue StandardError => e
        Puppet.err "Error updating java_home in #{file}: #{e.message}"
      end
    else
      message = "Skipping change of java_home in #{file}, because file not found."
      if warning
        Puppet.warning message
      else
        Puppet.debug message
      end
    end
  end
  # rubocop: enable Style/OptionalBooleanParameter

  # Assesses a list of detected Java home paths and determines the predominant one.
  # Issues warnings or errors if multiple paths are found.
  #
  # @param homes [Array<String>] An array of Java home paths detected across files.
  # @return [String, nil] The single, most frequent Java home path. Returns nil if
  #   the input is empty or if there's an unresolvable tie for the most frequent path.
  def self.assess_homes(homes)
    return nil if homes.nil? || homes.empty? # Handle empty input explicitly

    # Count occurrences of each value
    counts = homes.each_with_object(Hash.new(0)) { |val, hash| hash[val] += 1 }

    # If only one unique value exists, return it
    return homes.first if counts.size == 1

    # Find the maximum count
    max_count = counts.values.max

    # Find values with the maximum count
    most_frequent = counts.select { |_, count| count == max_count }.keys

    # If multiple values are tied for the most frequent
    fail "Multiple values for java_home found with the same highest frequency: #{most_frequent.join(', ')} in list #{homes.join(', ')}. Cannot determine predominant value." if most_frequent.size > 1

    predominant_home = most_frequent.first
    Puppet.warning "Multiple distinct values of java_home found: #{homes.uniq.join(', ')}. Using most frequent value as current value: #{predominant_home}"
    predominant_home
  end

  #
  # This method is called by Puppet when the resource needs to be modified
  # (i.e., the detected `java_home` differs from the desired state).
  # It iterates through the `DEFAULT_FILES` and any specified `extra_files`,
  # calling {#replace_java_home} for each to update the Java home path.
  #
  # @return [nil]
  #
  on_modify do
    # Ensure provider has a java_home value before attempting modifications
    unless provider.java_home && !provider.java_home.empty?
      Puppet.warning "Cannot modify java_home as current value is unknown or empty for resource #{self[:name]}."
      return nil
    end

    Puppet.debug "Starting java_home assessment for #{self[:name]}"
    DEFAULT_WLS_JAVA_HOME_FILES.each do |file_suffix|
      replace_java_home(File.join(self[:name], file_suffix))
    end
    # Ensure extra_files is an array even if nil or single string is provided
    Array(self[:extra_files]).each do |f|
      replace_java_home(File.join(self[:name], f), true) # Issue warning if extra file not found
    end
    Puppet.debug "Completed java_home modifications for #{self[:name]}"
    nil
  end

  # Parameters and Properties

  # @!attribute name
  # The namevar for the resource. This should be the absolute path to the
  # WebLogic home directory (e.g., Middleware Home or Domain Home) where
  # the configuration files reside.
  # @return [String]
  parameter :name

  # @!attribute extra_files
  # An optional list of additional file paths (relative to the `name` directory)
  # where the Java home should also be checked and potentially replaced.
  # Use this for non-standard configuration files.
  # @return [Array<String>]
  parameter :extra_files

  # @!attribute java_home
  # The desired JAVA_HOME path that should be configured in the WebLogic files.
  # Puppet will compare the detected Java home against this value. If they differ,
  # Puppet will attempt to replace the detected value with this desired value
  # in the relevant configuration files.
  # @return [String]
  property :java_home
end
