#
# wls_install::storeuserconfig
#
# @summary This defined type allows you to create a user config for WLST , this way you don't need to know the weblogic password.
# Here is an example on how you could use this:
# 
# ```puppet
# wls_install::storeuserconfig{'Wls12c':
#   domain_name                => "Wls12c",
#   adminserver_address        => "localhost",
#   adminserver_port           => 7001,
#   weblogic_password          => "weblogic1",
#   user_config_dir            => '/home/oracle',
#   logoutput                 => false,
# }
# ```
#
# @param [String[1]] domain_name
#    The domain name.
#
# @param weblogic_home_dir
#    Directory to use as WebLogic home.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [String[1]] adminserver_address
#    The address of the admin server.
#    May be a DNS name or a physical IP address.
#
# @param [Integer] adminserver_port
#    The IP port the admin server is listening on.
#
# @param [Stdlib::Absolutepath] user_config_dir
#    Directory where to store the user configs.
#
# @param [String[1]] weblogic_user
#    The base WebLogic username to configure.
#
# @param [Easy_type::Password] weblogic_password
#    The password of the base WebLogic user name.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Variant[Boolean, Enum['on_failure']]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::storeuserconfig (
  # Required parameters (alphabetically)
  Stdlib::Absolutepath          $user_config_dir,
  String[1]                     $adminserver_address         = lookup({ name => 'domain_adminserver_address' }),
  String[1]                     $domain_name                 = lookup({ name => 'domain_name' }),
  Stdlib::Absolutepath          $download_dir                = lookup({ name => 'wls_download_dir' }),
  Stdlib::Absolutepath          $jdk_home_dir                = lookup({ name => 'wls_jdk_home_dir' }),
  String[1]                     $os_group                    = lookup({ name => 'wls_os_group' }),
  String[1]                     $os_user                     = lookup({ name => 'wls_os_user' }),
  Stdlib::Absolutepath          $middleware_home_dir         = lookup({ name => 'wls_middleware_home_dir' }),
  Easy_type::Password           $weblogic_password           = lookup({ name => 'domain_wls_password' }),
  String[1]                     $weblogic_user               = lookup({ name => 'wls_weblogic_user', default_value => 'weblogic' }),

  # Optional parameters (alphabetically)
  Optional[Integer]             $administration_port         = undef,
  Boolean                       $administration_port_enabled = false,
  Integer                       $adminserver_port            = lookup({ name => 'domain_adminserver_port', default_value => 7001 }),
  Optional[Integer]             $adminserver_ssl_port        = undef,
  Boolean                       $custom_trust                = lookup({ name => 'wls_custom_trust', default_value => false }),
  Boolean                       $jsse_enabled                = lookup({ name => 'wls_jsse_enabled', default_value => false }),
  Optional[String[1]]           $trust_keystore_file         = lookup({ name => 'wls_trust_keystore_file', default_value => undef }),
  Optional[Easy_type::Password] $trust_keystore_passphrase   = lookup({ name => 'wls_trust_keystore_passphrase', default_value => undef }),
  Optional[String[1]]           $key_file_name               = undef,
  Optional[String[1]]           $config_file_name            = undef,
  Variant[Boolean, Enum['on_failure']] $logoutput            = lookup({ name => 'logoutput', default_value => 'on_failure' }),
) {
  # Consider adding validation for SSL configuration
  if $administration_port_enabled and $administration_port == undef {
    fail('administration_port must be set when administration_port_enabled is true')
  }

  # Add additional validations:
  if $custom_trust {
    if $trust_keystore_file == undef {
      fail('trust_keystore_file must be set when custom_trust is true')
    }
    if $trust_keystore_passphrase == undef {
      fail('trust_keystore_passphrase must be set when custom_trust is true')
    }
  }

  $key_file = if $key_file_name == undef {
    "${user_config_dir}/${os_user}-${domain_name}-WebLogicKey.properties"
  } else {
    "${user_config_dir}/${key_file_name}"
  }

  $config_file = if $config_file_name == undef {
    "${user_config_dir}${os_user}-${domain_name}-/WebLogicConfig.properties"
  } else {
    "${user_config_dir}/${config_file_name}"
  }

  # 
  #  Determine the configuration options for the trust keystore.
  # 
  $env = if $custom_trust {
    "CONFIG_JVM_ARGS=-Dweblogic.security.SSL.ignoreHostnameVerification=true -Dweblogic.ssl.JSSEEnabled=${jsse_enabled} -Dweblogic.security.SSL.enableJSSE=${jsse_enabled}  -Dweblogic.management.confirmKeyfileCreation=true -Dweblogic.security.TrustKeyStore=CustomTrust -Dweblogic.security.CustomTrustKeyStoreFileName=${trust_keystore_file} -Dweblogic.security.CustomTrustKeystorePassPhrase=${trust_keystore_passphrase.unwrap}"
  } else {
    "CONFIG_JVM_ARGS=-Dweblogic.security.SSL.ignoreHostnameVerification=true -Dweblogic.ssl.JSSEEnabled=${jsse_enabled} -Dweblogic.security.SSL.enableJSSE=${jsse_enabled}  -Dweblogic.management.confirmKeyfileCreation=true"
  }
  debug "Using ${env}"

  #
  # Depending on the options we specify we need to connect to a different URL for management 
  # of the domain.
  #
  $connect_url = if $administration_port_enabled == true {
    "t3s://${adminserver_address}:${administration_port}"
  } elsif $administration_port_enabled == false and $adminserver_ssl_port != undef {
    "t3s://${adminserver_address}:${adminserver_ssl_port}"
  } else {
    "t3://${adminserver_address}:${adminserver_port}"
  }

  $system_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

  easy_type::debug_evaluation() # Show local variable on extended debug

  # the py script used by the wlst*-
  file { "${download_dir}/${title}storeUserConfig.py":
    ensure    => file,
    path      => "${download_dir}/${title}storeUserConfig.py",
    content   => epp('wls_install/wlst/storeUserConfig.py.epp', {
        'key_file'      => $key_file,
        'config_file'   => $config_file,
        'weblogic_user' => $weblogic_user,
        'connect_url'   => $connect_url,
    }),
    backup    => false,
    show_diff => false,
    replace   => true,
    mode      => '0555',
    owner     => $os_user,
    group     => $os_group,
  }

  -> exec { "execwlst ${title}storeUserConfig.py":
    command     => "${middleware_home_dir}/oracle_common/common/bin/wlst.sh ${download_dir}/${title}storeUserConfig.py ${weblogic_password.unwrap}",
    environment => $env,
    creates     => $key_file,
    path        => $system_path,
    user        => $os_user,
    group       => $os_group,
    logoutput   => $logoutput,
  }
}
