#
# wls_install::utils::fmwcluster
#
# @summary This defined type allows you to convert an existing cluster into a OSB or SOA suite cluster (BPM is optional) and also convert BAM to a BAM cluster.
# This will also work for OIM / OAM cluster. Here is an example on how you can use this:
# 
# ```puppet
# wls_install::utils::fmwcluster { 'soaCluster':
#   domain_name         => "soa_domain",
#   middleware_home_dir => "/opt/oracle/middleware12",
#   jdk_home_dir        => "/usr/java/jdk1.8.0_231",
#   wls_domains_dir     => '/opt/oracle/domains',
#   weblogic_user       => 'weblogic',
#   weblogic_password   => Sensitive('welcome1'),
#   adminserver_address => "app.soa.com",
#   adminserver_port    => 7001,
#   soa_cluster_name    => "SoaCluster",
#   soa_enabled         => true,
#   os_user             => 'weblogic',
#   os_group            => 'oracle',
#   download_dir        => '/tmp',
#   logoutput           => true,
# }
# ```
#
# @param [Wls_install::Versions] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Stdlib::Absolutepath] middleware_home_dir
#    Middleware home directory.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [Stdlib::Absolutepath] wls_domains_dir
#    The root directory where all domain directories reside in.
#
# @param [String[1]] adminserver_name
#    The name of the Admin Server.
#
# @param [String[1]] domain_name
#    The domain name.
#
# @param [String[1]] adminserver_address
#    The address of the admin server.
#    May be a DNS name or a physical IP address.
#
# @param [Integer] adminserver_port
#    The IP port the admin server is listening on.
#
# @param [Optional[Integer]] administration_port
#    The common secure administration port for this WebLogic Server domain.
#    (Requires you to enable the administration port.)
#
# @param [Boolean] administration_port_enabled
#    Specifies whether the domain-wide administration port should be enabled for this WebLogic Server domain.
#    Because the administration port uses SSL, enabling the administration port requires that SSL must be configured for all servers in the domain.
#    The domain-wide administration port enables you to start a WebLogic Server instance in STANDBY state. It also allows you to separate administration traffic from application traffic in your domain. Because all servers in the domain must enable or disable the administration port at once, you configure the default administration port settings at the domain level.
#    If you enable the administration port:
#    The administration port accepts only connections that specify administrator credentials.
#    Connections that specify administrator credentials can use only the administration port.
#    The command that starts managed servers must specify a secure protocol and the administration port: -Dweblogic.management.server=https://admin_server:administration_port
#
# @param [Optional[Integer]] adminserver_ssl_port
#    SSL port to use for the Admin server.
#
# @param [String[1]] weblogic_user
#    The base WebLogic username to configure.
#
# @param [Easy_type::Password] weblogic_password
#    The password of the base WebLogic user name.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Boolean] jsse_enabled
#    Determine if you want to enable JSSE security.
#
# @param [Boolean] custom_trust
#    Determine if you want to use a custom trust or not.
#
# @param [Optional[String[1]]] trust_keystore_file
#    File specificiation of the trust keystore.
#
# @param [Optional[Easy_type::Password]] trust_keystore_passphrase
#    Passphrase to use for the trust keystore.
#
# @param [Optional[String[1]]] soa_cluster_name
#    Name of the cluster to target SOA managed servers on.
#
# @param [Optional[String[1]]] bam_cluster_name
#    Name of the cluster to target BAM managed servers on.
#
# @param [Optional[String[1]]] osb_cluster_name
#    Name of the cluster to target OSB managed servers on.
#
# @param [Optional[String[1]]] oam_cluster_name
#    Name of the cluster to target OAM managed servers on.
#
# @param [Optional[String[1]]] oim_cluster_name
#    Name of the cluster to target OIM managed servers on.
#
# @param [Optional[String[1]]] ess_cluster_name
#    Name of the cluster to target ESS managed servers on.
#
# @param [Optional[String[1]]] bi_cluster_name
#    Name of the cluster to target BI managed servers on.
#
# @param [Boolean] bpm_enabled
#    Boolean to determine if bpm is enabled.
#
# @param [Boolean] bam_enabled
#    Boolean to determine if bam is enabled.
#
# @param [Boolean] osb_enabled
#    Boolean to determine if osb is enabled.
#
# @param [Boolean] soa_enabled
#    Boolean to determine if soa is enabled.
#
# @param [Boolean] oam_enabled
#    Boolean to determine if oam is enabled.
#
# @param [Boolean] oim_enabled
#    Boolean to determine if oim is enabled.
#
# @param [Boolean] b2b_enabled
#    Boolean to determine if b2b is enabled.
#
# @param [Boolean] ess_enabled
#    Boolean to determine if ess is enabled.
#
# @param [Boolean] bi_enabled
#    Boolean to determine if bi is enabled.
#
# @param [Variant[Boolean, Enum['on_failure', ]]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::utils::fmwcluster (
  Wls_install::Versions          $version                     = lookup({ name => 'wls_version', default_value => 12214 }),
  Stdlib::Absolutepath           $middleware_home_dir         = lookup({ name => 'wls_middleware_home_dir' }),
  Stdlib::Absolutepath           $jdk_home_dir                = lookup({ name => 'wls_jdk_home_dir' }),
  Stdlib::Absolutepath           $wls_domains_dir             = lookup({ name => 'wls_domains_dir', default_value => undef }),
  String[1]                      $domain_name                 = lookup({ name => 'domain_name' }),
  String[1]                      $adminserver_name            = lookup({ name => 'domain_adminserver', default_value => 'AdminServer' }),
  String[1]                      $adminserver_address         = lookup({ name => 'domain_adminserver_address' }),
  Integer                        $adminserver_port            = lookup({ name => 'domain_adminserver_port', default_value => 7001 }),
  Optional[Integer]              $administration_port         = undef,
  Boolean                        $administration_port_enabled = false,
  Optional[Integer]              $adminserver_ssl_port        = undef,
  String[1]                      $weblogic_user               = lookup({ name => 'wls_weblogic_user', default_value => 'weblogic' }),
  Easy_type::Password            $weblogic_password           = lookup({ name => 'domain_wls_password', default_value => undef }),
  String[1]                      $os_user                     = lookup({ name => 'wls_os_user' }),
  String[1]                      $os_group                    = lookup({ name => 'wls_os_group' }),
  Stdlib::Absolutepath           $download_dir                = lookup({ name => 'wls_download_dir' }),
  Boolean                        $jsse_enabled                = lookup({ name => 'wls_jsse_enabled', default_value => false }),
  Boolean                        $custom_trust                = lookup({ name => 'wls_custom_trust', default_value => false }),
  Optional[String[1]]            $trust_keystore_file         = lookup({ name => 'wls_trust_keystore_file', default_value => undef }),
  Optional[Easy_type::Password]  $trust_keystore_passphrase   = lookup({ name => 'wls_trust_keystore_passphrase', default_value => undef }),
  Optional[String[1]]            $soa_cluster_name            = undef,
  Optional[String[1]]            $bam_cluster_name            = undef,
  Optional[String[1]]            $osb_cluster_name            = undef,
  Optional[String[1]]            $oam_cluster_name            = undef,
  Optional[String[1]]            $oim_cluster_name            = undef,
  Optional[String[1]]            $ess_cluster_name            = undef,
  Optional[String[1]]            $bi_cluster_name             = undef,
  Boolean                        $bpm_enabled                 = false,
  Boolean                        $bam_enabled                 = false,
  Boolean                        $osb_enabled                 = false,
  Boolean                        $soa_enabled                 = false,
  Boolean                        $oam_enabled                 = false,
  Boolean                        $oim_enabled                 = false,
  Boolean                        $b2b_enabled                 = false,
  Boolean                        $ess_enabled                 = false,
  Boolean                        $bi_enabled                  = false,
  Variant[
    Boolean,
    Enum[
      'on_failure',
    ]
  ]                              $logoutput                  = lookup({ name => 'logoutput', default_value => 'on_failure' }),

) {
  # 
  # This script is only tested and works on OFM 12.1.4 and higher.
  # 
  # lint:ignore:version_comparison
  if ( $version < 12214 ) {
    fail("Unsupported version ${version} for this module")
  }
  # lint:endignore:version_comparison
  # 
  #  Determine the configuration options for the trust keystore.
  # 
  $env = if $custom_trust {
    "CONFIG_JVM_ARGS=-Dweblogic.security.SSL.ignoreHostnameVerification=true -Dweblogic.ssl.JSSEEnabled=${jsse_enabled} -Dweblogic.security.SSL.enableJSSE=${jsse_enabled} -Dweblogic.security.TrustKeyStore=CustomTrust -Dweblogic.security.CustomTrustKeyStoreFileName=${trust_keystore_file} -Dweblogic.security.CustomTrustKeystorePassPhrase=${trust_keystore_passphrase.unwrap}"
  } else {
    "CONFIG_JVM_ARGS=-Dweblogic.security.SSL.ignoreHostnameVerification=true -Dweblogic.ssl.JSSEEnabled=${jsse_enabled} -Dweblogic.security.SSL.enableJSSE=${jsse_enabled}"
  }
  debug "Using ${env}"

  #
  # Depending on the options we specify we need to connect to a different URL for management 
  # of the domain.
  #
  $connect_url = if $administration_port_enabled == true {
    "t3s://${adminserver_address}:${administration_port}"
  } elsif $administration_port_enabled == false and $adminserver_ssl_port != undef {
    "t3s://${adminserver_address}:${adminserver_ssl_port}"
  } else {
    "t3://${adminserver_address}:${adminserver_port}"
  }

  $domain_dir = "${wls_domains_dir}/${domain_name}"

  if ( $soa_enabled ) {
    # check if the soa is already targeted to the cluster on this weblogic domain
    if ( wls_install::soa_cluster_configured($domain_dir, $soa_cluster_name) ) {
      $convert_soa = false
    } else {
      $convert_soa = true
    }
  } else {
    $convert_soa = false
  }

  if ( $osb_enabled ) {
    # check if the osb is already targeted to the cluster on this weblogic domain
    if ( wls_install::osb_cluster_configured($domain_dir, $osb_cluster_name) ) {
      $convert_osb = false
    } else {
      $convert_osb = true
    }
  } else {
    $convert_osb = false
  }

  if ( $bam_enabled ) {
    # check if the bam is already targeted to the cluster on this weblogic domain
    if ( wls_install::bam_cluster_configured($domain_dir, $bam_cluster_name) ) {
      $convert_bam = false
    } else {
      $convert_bam = true
    }
  } else {
    $convert_bam = false
  }

  easy_type::debug_evaluation() # Show local variable on extended debug

  if ( $convert_soa or $convert_osb or $convert_bam ) {
    $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

    file { "${download_dir}/retargetAllToCluster${title}.py":
      ensure  => file,
      content => epp('wls_install/wlst/wlstexec/fmw/retargetAlltoCluster.py.epp', {
          'weblogic_user'     => $weblogic_user,
          'weblogic_password' => $weblogic_password,
          'admin_server_name' => $adminserver_name,
          'bpm_enabled'       => $bpm_enabled,
          'bam_enabled'       => $bam_enabled,
          'soa_enabled'       => $soa_enabled,
          'osb_enabled'       => $osb_enabled,
          'b2b_enabled'       => $b2b_enabled,
          'ess_enabled'       => $ess_enabled,
          'bi_enabled'        => $bi_enabled,
          'oam_enabled'       => $oam_enabled,
          'oim_enabled'       => $oim_enabled,
          'soa_cluster_name'  => $soa_cluster_name,
          'bam_cluster_name'  => $bam_cluster_name,
          'osb_cluster_name'  => $osb_cluster_name,
          'ess_cluster_name'  => $ess_cluster_name,
          'bi_cluster_name'   => $bi_cluster_name,
          'oam_cluster_name'  => $oam_cluster_name,
          'oim_cluster_name'  => $oim_cluster_name,
          'connect_url'       => $connect_url,
      }),
      backup  => false,
      replace => true,
      mode    => '0775',
      owner   => $os_user,
      group   => $os_group,
    }

    # retarget all apps,libraries, startup , shutdown and datasources
    -> exec { "execwlst retargetAllToCluster.py ${title}":
      environment => $env,
      command     => "${middleware_home_dir}/oracle_common/common/bin/wlst.sh ${download_dir}/retargetAllToCluster${title}.py",
      path        => $exec_path,
      user        => $os_user,
      group       => $os_group,
      timeout     => 0,
      logoutput   => $logoutput,
    }
    cleanup { "${download_dir}/retargetAllToCluster${title}.py": }
  }
}
